@@ -1,9 +1,9 @@
 /*
- * Copyright(c) 1997,1998,1999 - All Rights Reserved
+ * Copyright(c) 1997-2000 - All Rights Reserved
  *
  * See the COPYRIGHT file.
  *
- * $Id: acconfig.h,v 1.7 1999/01/02 22:14:20 kalt Exp $
+ * $Id: acconfig.h,v 1.9 2000/07/31 22:33:54 kalt Exp $
  *
  */
 
@@ -27,3 +27,5 @@
 /* Define if regular expression matching functions are there */
 #undef HAVE_REGEXP
 
+/* Define if we're stuck with GNU Pth */
+#undef GNU_PTH
