@@ -1,13 +1,13 @@
 /*
  * commands.c: all the client commands.
  *
- * Copyright(c) 1997,1998,1999 - All Rights Reserved
+ * Copyright(c) 1997-2000 - All Rights Reserved
  *
  * See the COPYRIGHT file.
  */
 
 #ifndef lint
-static char rcsid[] = "@(#)$Id: commands.c,v 1.62 1999/08/02 21:37:12 kalt Exp $";
+static char rcsid[] = "@(#)$Id: commands.c,v 1.64 2000/07/31 22:33:54 kalt Exp $";
 #endif
 
 #include "os.h"
@@ -472,7 +472,7 @@ cmd_dns(p)
   if (*p)
     {
       vsic_slog(LOG_CLIENT, "--- Looking up \"%s\" in DNS", p);
-      dns_lookup(p);
+      dns_lookup(p, NULL);
       return 0;
     }
   else
