@@ -1,30 +1,21 @@
 /*
  * server.c: interactions with servers, tcp code, and DNS resolver
  *
- * Copyright(c) 1997,1998,1999 - All Rights Reserved
+ * Copyright(c) 1997-2000 - All Rights Reserved
  *
  * See the COPYRIGHT file.
  */
 
 #ifndef lint
-static char rcsid[] = "@(#)$Id: server.c,v 1.65 1999/08/02 21:49:34 kalt Exp $";
+static char rcsid[] = "@(#)$Id: server.c,v 1.74 2000/08/01 20:07:34 kalt Exp $";
 #endif
 
 #include "os.h"
 
-#include <sys/socket.h>
-#include <netinet/in.h>
-#include <arpa/inet.h>
-#include <netdb.h>
 #if HAVE_SYS_FILIO_H
 # include <sys/filio.h>
 #endif
 
-#ifdef BIND_HATRED
-# undef inet_ntoa
-# undef inet_addr
-#endif
-
 #include "struct.h"
 #include "term.h"
 #include "window.h"
@@ -41,261 +32,64 @@ extern void sic_format(char *, char *, char *, char *);
 static struct server_	*slist = NULL;
 struct server_		*server = NULL;
 
-static int		dns_pid = -1, dns_pipe[2], dns_check = 0;
-
-static void sic_connect(struct server_ *);
-
-/* dns_init: start the slave DNS resolver. */
-static void
-dns_init()
-{
-  static int first = 1;
-  int i;
-  int smoke[2], ekoms[2];
-
-  if (first)
-    {
-      signal(SIGPIPE, SIG_IGN);
-      signal(SIGCHLD, SIG_IGN); /* to avoid zombies */
-    }
-
-  dns_check = 0;
-  if (pipe(smoke) < 0)
-    {
-      term_end();
-      perror("pipe: ");
-      printf("Unable to start DNS resolver\n");
-      exit(1);
-    }
-  if (pipe(ekoms) < 0)
-    {
-      term_end();
-      perror("pipe: ");
-      printf("Unable to start DNS resolver\n");
-      exit(1);
-    }
-  dns_pipe[0] = smoke[0];
-  dns_pipe[1] = ekoms[1];
-  dns_pid = 0;
-
-  switch (i = fork())
-    {
-  case -1:
-      sic_slog(LOG_CLIENT, "--- fork() failed!");
-      break;
-  case 0:
-      /* child */
-      close(0);
-      close(1);
-      close(2);
-      dup(ekoms[0]);
-      dup(smoke[1]);
-      close(smoke[0]); close(smoke[1]);
-      close(ekoms[0]); close(ekoms[1]);
-      execlp("./sdns", "./sdns (sic slave)", NULL);
-      if (execlp("sdns", "sdns (sic slave)", NULL) < 0)
-	  _exit(0);
-      abort();
-  default:
-      /* parent */
-      dns_pid = i;
-      close(smoke[1]); close(ekoms[0]);
-      break;
-    }
-}
-
-/* dns_restart: to be called if the slave isn't running
- *	calls dns_init() if reasonnable to attempt to (re)start the slave
- *	if successful, it will retry outstanding server lookups.
- */
-static void
-dns_restart(what)
-  char *what;
-{
-  static time_t restart_t = 0;
-  static char loop = 0;
-
-  if (loop)
-      return;
-  loop = 1;
-  dns_pid = 0;
-  if ((time(NULL) - restart_t) > 15)
-    {
-      dns_init();
-      restart_t = time(NULL);
-      if (write(dns_pipe[1], what, strlen(what)+1) < 0)
-	  sic_slog(LOG_CLIENT,"--- Unable to restart slave resolver.");
-      else
-	{
-	  struct server_ *stmp = slist;
-
-	  sic_slog(LOG_CLIENT, "--- Restarted slave resolver.");
-	  dns_check++;
-	  while (stmp)
-	    {
-	      if (option(stmp->sopt, S_LOOKUP))
-		  dns_lookup(stmp->sname);
-	      stmp = stmp->nexts;
-	    }
-	}
-    }
-  loop = 0;
-}
-
-/* dns_lookup: performs async DNS lookup for the hostname/IP "what" */
-void
-dns_lookup(what)
-  char *what;
-{
-  vsic_slog(LOG_DEBUG, "DNS: looking up \"%s\" (%d)", what, 1+dns_check);
-
-  if (dns_pid == -1)
-    {
-      dns_init();
-      if (dns_pid == 0)
-	{
-	  sic_slog(LOG_CLIENT, "--- Unable to start slave resolver!");
-	  return;
-	}
-    }
-
-  if (dns_pid > 0)
-      /* slave DNS should be here */
-      if (write(dns_pipe[1], what, strlen(what)+1) < 0)
-	{
-	  int ret = 0;
-	  
-	  vsic_slog(LOG_DEBUG, "DNS: write() failed: %s", strerror(errno));
-	  if (dns_pid > 0 && (ret = kill(dns_pid, SIGKILL)))
-	    {
-	      vsic_slog(LOG_DEBUG, "DNS: kill() returned %d", ret);
-	      if (ret == ESRCH)
-		  sic_slog(LOG_CLIENT, "--- Slave resolver process is dead.");
-	    }
-	  else
-	      sic_slog(LOG_CLIENT,
-		       "--- Slave resolver process not responding!");
-	  
-	  vsic_slog(LOG_DEBUG, "DNS: kill()'ed %d", dns_pid);
-	  dns_restart(what);
-	}
-      else
-	  dns_check++;
-  else
-      dns_restart(what);
-}
-
-/* dns_parse: parse a reply from the slave DNS resolver and use it */
-static void
-dns_parse()
-{
-  char buf[4096], *wb = buf, *wp;
-  int i, j = 0;
-
-  i = read(dns_pipe[0], buf, 4096);
-  while (j < i)
-    {
-      struct server_ *stmp = slist;
-      int rc = 0;
-
-      if ((wp = index(wb, '=')) == NULL)
-	{
-	  wp = index(wb, ':');
-	  rc = -1;
-	}
-      assert(wp);
-      *wp++ = '\0';
-      vsic_slog(LOG_DEBUG, "DNS: answer \"%s\"", wb);
-	  
-      while (stmp)
-	{
-	  if (option(stmp->sopt, S_DNS)
-	      && (!strcasecmp(stmp->sname, wb)
-		  || !strcasecmp((stmp->ip) ? stmp->ip : "?" , wb)))
-	      break;
-	  stmp = stmp->nexts;
-	}
-      if (rc == 0)
-	  if (stmp)
-	      if (stmp->ip)
-		{
-		  free(stmp->sname);
-		  stmp->sname = strdup(wp);
-		}
-	      else
-		{
-		  stmp->ip = strdup(wp);
-		  unset_option(stmp->sopt, S_LOOKUP);
-		  unset_option(stmp->sopt, S_DNS);
-		  if (option(stmp->sopt, S_CONNECTING))
-		    {
-		      vsic_slog(LOG_CLIENT, "--- Connecting to %s %d [%s]",
-				stmp->sname, stmp->port, stmp->ip);
-		      server = stmp;
-		      sic_connect(stmp);
-		    }
-		}
-	  else
-	      vsic_slog(LOG_CLIENT, "--- DNS: %s is %s", wb, wp);
-      else /* rc == -1 */
-	{
-	  rc = atoi(wp);
-
-	  vsic_slog(LOG_CLIENT, "--- %s: \"%s\" %s",
-		    (stmp) ?
-		    (option(stmp->sopt, S_CONNECTING)) ? "Connect failed":
-		    "Server lookup failed" : "DNS lookup",
-		    wb, ((rc = atoi(wp)) == HOST_NOT_FOUND) ? "not found." :
-		    (rc == TRY_AGAIN) ? "not found, try again." :
-		    (rc == NO_DATA) ? "has no IP address." : wp);
-	  if (stmp)
-	    {
-	      unset_option(stmp->sopt, S_LOOKUP);
-	      unset_option(stmp->sopt, S_DNS);
-	      if (option(stmp->sopt, S_CONNECTING))
-		  unset_option(stmp->sopt, S_CONNECTING);
-	    }
-	}
-      *(wp-1) = ' ';
-      j += strlen(wb)+1;
-      assert( buf[j-1] == '\0' );
-      wb = buf+j;
-      dns_check--;
-    }
-}
-
 /* sic_connect: connect() to a server (given a struct server_ pointer) */
-static void
+void
 sic_connect(srv)
   struct server_ *srv;
 {
   struct  sockaddr_in	sock_sin;
-  int			fd, opt = 1;
+  int			fd, opt = 1, rc;
   char			*vif;
+#if defined(HAVE_GETADDRINFO)
+  struct  sockaddr	sa;
 
-  assert (srv->ip);
+  if (srv->address)
+      bcopy((char *)srv->address->ai_addr, (char *)&sa,
+	    srv->address->ai_addrlen);
+#endif
+
+  assert (srv->ip);	/* even if srv->address is defined */
 
   bzero((char *)&sock_sin, sizeof(sock_sin));
   sock_sin.sin_family = AF_INET;
 
-  if ((fd = socket(AF_INET, SOCK_STREAM, 0)) < 0)
+#if defined(HAVE_GETADDRINFO)
+  if (srv->address)
+      fd = socket(srv->address->ai_family, SOCK_STREAM,
+		  srv->address->ai_protocol);
+  else
+#endif
+      fd = socket(AF_INET, SOCK_STREAM, 0);
+
+  if (fd < 0)
       vsic_slog(LOG_CLIENT, "--- socket() call failed: %s", strerror(errno));
   if (fd > 0 && (vif = getenv("SICIP")))
     {
+      /* the following should be fixed rather than simply reporting an error */
       sock_sin.sin_addr.s_addr = inet_addr(vif);
-      if (bind(fd, (struct sockaddr *) &sock_sin, sizeof(sock_sin)) < 0)
-	  vsic_slog(LOG_CLIENT, "--- Unable to bind to IP %s: %s", vif,
-		    strerror(errno));
+      if (sock_sin.sin_addr.s_addr == INADDR_NONE)
+	  vsic_slog(LOG_CLIENT, "--- Bad IP address: SICIP = %s", vif);
+      else
+	  if (bind(fd, (struct sockaddr *) &sock_sin, sizeof(sock_sin)) < 0)
+	      vsic_slog(LOG_CLIENT, "--- Unable to bind to IP %s: %s", vif,
+			strerror(errno));
     }
   if (fd > 0 && ioctl(fd, FIONBIO, &opt) < 0)
       vsic_slog(LOG_CLIENT, "--- ioctl() for %s:%d failed: %s", srv->sname,
 		srv->port, strerror(errno));
 
-  sock_sin.sin_port = htons(srv->port);
-  sock_sin.sin_addr.s_addr = inet_addr(srv->ip);
+#if defined(HAVE_GETADDRINFO)
+  if (srv->address)
+      rc = connect(fd, &sa, srv->address->ai_addrlen);
+  else
+#endif
+    {
+      sock_sin.sin_port = htons(srv->port);
+      sock_sin.sin_addr.s_addr = inet_addr(srv->ip);
+      rc = connect(fd, (struct sockaddr *)&sock_sin, sizeof(sock_sin));
+    }
 
-  if (fd > 0 && connect(fd, (struct sockaddr *)&sock_sin, sizeof(sock_sin)) < 0
-      && errno != EINPROGRESS)
+  if (fd > 0 && rc < 0 && errno != EINPROGRESS)
     {
       vsic_slog(LOG_CLIENT, "--- Connection to %s:%d failed: %s", srv->sname,
 		srv->port, strerror(errno));
@@ -414,17 +208,35 @@ sic_server(nick, name, port, pass)
   *stmp = server;
 
   server->nick = strdup(nick);
-  if (isdigit((int) *name))
-      server->ip = strdup(name);
   server->sname = strdup(name);
   server->port = port;
   if (pass)
       server->pass = strdup(pass);
 
+#if defined(HAVE_GETADDRINFO)
+  {
+    struct addrinfo *address, hints;
+    char port_str[10];
+    
+    bzero((char *)&hints, sizeof(hints));
+    hints.ai_flags = AI_CANONNAME|AI_NUMERICHOST;
+    hints.ai_family = PF_UNSPEC;
+    hints.ai_socktype = SOCK_STREAM;
+    sprintf(port_str, "%d", port);
+    if (getaddrinfo(name, port_str, &hints, &address) == 0)
+      {
+	server->ip = strdup(name);
+	server->address = address;
+      }
+  }
+#else
+  if (inet_addr(name) != INADDR_NONE)
+      server->ip = strdup(name);
+#endif
+
   if (server->ip == NULL)
     {
-      dns_lookup(name);
-      set_option(server->sopt, S_LOOKUP);
+      dns_lookup(name, server);
       set_option(server->sopt, S_DNS);
     }
 
@@ -657,17 +469,19 @@ sic_select()
 	  FD_SET(stmp->fd, &wfd);
       if (option(stmp->sopt, S_LISTEN))
 	  FD_SET(stmp->fd, &rfd);
-      if (option(stmp->sopt, S_CONNECTING) && !option(stmp->sopt, S_LOOKUP))
+      if (option(stmp->sopt, S_CONNECTING) && !option(stmp->sopt, S_DNS))
 	  FD_SET(stmp->fd, &wfd);
       stmp = stmp->nexts;
     }
-  if (dns_pid > 0 && dns_check)
-      FD_SET(dns_pipe[0], &rfd);
 
   display_status();
   term_input(NULL, 0);
   term_flush();
+#if defined(GNU_PTH)
+  n = pth_select(255, &rfd, &wfd, NULL, &timeout);
+#else
   n = select(255, &rfd, &wfd, NULL, &timeout);
+#endif
 
   if (n < 0)
       return 0;
@@ -705,7 +519,7 @@ sic_select()
 		  vsic_slog(LOG_CLIENT,
 			    "--- DCC connection received from %s@%s",
 			    stmp->nick, stmp->sname);
-		  dns_lookup(stmp->sname);
+		  dns_lookup(stmp->sname, NULL);
 		  sz = 1;
 		  if (ioctl(stmp->fd, FIONBIO, &sz) < 0)
 		      vsic_slog(LOG_CLIENT, "--- ioctl() failed: %s",
@@ -962,12 +776,6 @@ sic_select()
       stmp = stmp->nexts;
     }
   
-  if (dns_pid > 0 && FD_ISSET(dns_pipe[0], &rfd))
-    {
-      select_active(NULL, 2);
-      dns_parse();
-    }
-
   if (FD_ISSET(0, &rfd))
     return 1;
   else
@@ -983,6 +791,7 @@ cmd_server(p)
 
   if (p == NULL)
     {
+      /* NULL argument, this is only used internally */
       if (server == NULL)
 	  return 0;
       if (option(server->sopt, S_CONNECTING)
@@ -1004,8 +813,10 @@ cmd_server(p)
       set_option(server->sopt, S_QUIT);
       return 0;
     }
-  if (!*p)
+  /* real user input */
+  if (!*p || !strcasecmp(p, "-v"))
     {
+      /* no argument or just `-v', user is simply requestint a list */
       sic_slog(LOG_CLIENT, "--- Server list:");
       while (stmp)
 	{
@@ -1015,13 +826,34 @@ cmd_server(p)
 			(stmp->pass) ? "*password* " : "",
 			(option(stmp->sopt, S_CONNECTING)) ? "will be " :
 			(option(stmp->sopt, S_CONNECTED)) ? "" : "was ",
-			stmp->nick, (stmp->ip) ? "" : " Unknown IP address");
+			stmp->nick,
+			(stmp->ip) ? "" : option(stmp->sopt, S_DNS) ?
+				" Looking up address" : " Unknown address");
+	  if (*p && stmp->ip)
+#if defined(HAVE_GETADDRINFO)
+	      vsic_slog(LOG_CLIENT, "---       => %s[%s]",
+			(stmp->address && stmp->address->ai_canonname) ?
+			stmp->address->ai_canonname : "???", stmp->ip);
+#else
+	      vsic_slog(LOG_CLIENT, "---       => [%s]", stmp->ip);
+#endif
 	  stmp = stmp->nexts;
 	}
     }
   else
-      if (isdigit((int) *p) && !index(p, '.'))
+    {
+      char *number = p;
+
+      while (*number)
+	  if (!isdigit(*number++))
+	    {
+	      number = NULL;
+	      break;
+	    }
+
+      if (number)
 	{
+	  /* argument is a number, find selected server and try to connect */
 	  if (current->via && (option(current->via->sopt, S_CONNECTED)
 			       || option(current->via->sopt, S_CONNECTING)))
 	    {
@@ -1038,7 +870,7 @@ cmd_server(p)
 	  if (stmp)
 	    {
 	      server = stmp;
-	      if (stmp->ip || option(stmp->sopt, S_LOOKUP))
+	      if (stmp->ip || option(stmp->sopt, S_DNS))
 		{
 		  if (sic_swin(1))
 		    {
@@ -1053,7 +885,7 @@ cmd_server(p)
 		      else
 			{
 			  set_option(stmp->sopt, S_CONNECTING);
-			  if (!option(stmp->sopt, S_LOOKUP))
+			  if (!option(stmp->sopt, S_DNS))
 			    {
 			      vsic_slog(LOG_CLIENT,
 					"--- Connecting to %s %d [%s]",
@@ -1090,6 +922,7 @@ cmd_server(p)
 		     p, portn, pass);
 	  cmd_server("");
 	}
+    }
   return 0;
 }
 
@@ -1194,7 +1027,7 @@ sic_dcc(type, nick, ip, port, file, size)
   dtmp->sname = strdup((char *)inet_ntoa(*(struct in_addr*)&lip));
   dtmp->ip = strdup((char *)inet_ntoa(*(struct in_addr*)&lip));
   set_option(dtmp->sopt, S_NOTCONNECTED);
-  dns_lookup(dtmp->ip);
+  dns_lookup(dtmp->ip, NULL);
   switch (type)
     {
   case 0:
