@@ -1,11 +1,12 @@
 /*
- * Copyright(c) 1997,1998,1999 - All Rights Reserved
+ * Copyright(c) 1997-2000 - All Rights Reserved
  *
  * See the COPYRIGHT file.
  */
 
-void	dns_lookup(char *);
+void	dns_lookup(char *, struct server_ *);
 
+void	sic_connect(struct server_ *);
 struct server_	*select_server(char *);
 struct server_	*sic_server(char *, char *, int, char *);
 int	sic_select();
