@@ -1,13 +1,13 @@
 /*
  * sic.c: testing...
  *
- * Copyright(c) 1997,1998,1999 - All Rights Reserved
+ * Copyright(c) 1997-2000 - All Rights Reserved
  *
  * See the COPYRIGHT file.
  */
 
 #ifndef lint
-static char rcsid[] = "@(#)$Id: sic.c,v 1.21 1999/08/02 21:33:31 kalt Exp $";
+static char rcsid[] = "@(#)$Id: sic.c,v 1.24 2000/08/01 02:22:16 kalt Exp $";
 #endif
 
 #include "os.h"
@@ -20,6 +20,7 @@ static char rcsid[] = "@(#)$Id: sic.c,v 1.21 1999/08/02 21:33:31 kalt Exp $";
 #include "option.h"
 #include "config.h"
 #include "counter.h"
+#include "adns.h"
 
 extern const char version[];
 
@@ -49,6 +50,12 @@ int sig;
   last = time(NULL);
 }
 
+/* sic_sigign: SIG_IGN to allow SA_INTERRUPT to work */
+static void
+sic_sigign(sig)
+int sig;
+{}
+
 /* sic_signals: setup our own signal handlers */
 static void
 sic_signals()
@@ -66,6 +73,10 @@ sic_signals()
   sigaction(SIGCONT, &sa, NULL);
   sa.sa_handler = sic_sigint;
   sigaction(SIGINT, &sa, NULL);
+  sa.sa_handler = sic_sigign;
+  sigaction(SIGUSR1, &sa, NULL);
+  sa.sa_flags = 0;
+  sigaction(SIGPIPE, &sa, NULL);
 }
 
 /* main: you know, main! */
@@ -103,7 +114,7 @@ main (argc, argv)
 	startup = strdup(optarg); /* is strdup needed? */
 	break;
     case 'v':
-	printf("sic %s, Copyright (C) 1997,1998,1999 Christophe Kalt <kalt@stealth.net>\n", version);
+	printf("sic %s, Copyright (C) 1997-2000 Christophe Kalt <kalt@stealth.net>\n", version);
 #if defined(USE_GNU_RX)
 	printf("\tLinked with %s\n", rx_version_string);
 #endif
@@ -125,7 +136,7 @@ main (argc, argv)
     }
 
   /* no window/lastlog ability */
-  /* dns_init(); */
+  dns_init();		/* initialize the resolver thread */
   opt_init();		/* initialize global options */
   counter_init();	/* init counters */
   sic_signals();	/* setup signals */
@@ -200,6 +211,8 @@ main (argc, argv)
       if (sic_select() > 0)
 	  sic_getch();
 
+      dns_process();
+
       now = time(NULL);
 #if defined(HAVE_REGEXP)
       if (now > next_counter)
