@@ -1,7 +1,7 @@
 /*
  * 
  *
- * Copyright(c) 1997,1998,1999 - All Rights Reserved
+ * Copyright(c) 1997-2000 - All Rights Reserved
  *
  * See the COPYRIGHT file.
  */
@@ -192,8 +192,7 @@ enum /* server options */
 {
   S_DEFAULT = 0,
   S_CONFIG,		/* configuration file read */
-  S_LOOKUP,		/* looking up the host name in order to connect */
-  S_DNS,		/* doing DNS lookup */
+  S_DNS,		/* waiting for DNS lookup to give IP address */
   S_NOTCONNECTED,	/* not connected (irc wise: tcp conn might be up) */
   S_CONNECTING,		/* connecting (tcp wise) */
   S_CONNECTED,		/* connected (tcp wise) */
@@ -218,6 +217,9 @@ struct server_
 {
   char			*sname;
   char			*ip;
+#if defined(HAVE_GETADDRINFO)
+  struct addrinfo	*address;
+#endif
   char			*pass;		/* also used for DCC filenames */
   int			port;
   char			*nick;
