@@ -1,5 +1,5 @@
 /*
- * Copyright(c) 1997,1998,1999 - All Rights Reserved
+ * Copyright(c) 1997-2000 - All Rights Reserved
  *
  * See the COPYRIGHT file.
  */
@@ -45,7 +45,7 @@ void		term_putchar(unsigned int);
 int		term_puts(char *, int);
 int		term_putes(char *, int, u_char *, u_int);
 void		term_flush();
-void		term_scroll(int, int, int);
+int		term_scroll(int, int, int);
 void		term_status(char *, int);
 void		term_input(char *, int);
 
