@@ -1,5 +1,5 @@
 /*
- * Copyright(c) 1997,1998,1999 - All Rights Reserved
+ * Copyright(c) 1997-2000 - All Rights Reserved
  *
  * See the COPYRIGHT file.
  */
@@ -12,8 +12,11 @@ extern	int	rmatch(const char *, const char *);
 extern	char *	my_cftime(char *, const int len, const char *, const time_t);
 extern	char *	my_strftime(char *, const int len, const char *);
 extern	char *	sic_tdiff(time_t, char);
-#ifndef HAVE_STRERROR
+# if !defined(HAVE_STRERROR)
 extern	char *	strerror(int);
-#endif
+# endif
+# if defined(HAVE_GETADDRINFO)
+extern	char *	aitoip(struct addrinfo *);
+# endif
 
 #endif
