@@ -1,13 +1,13 @@
 /*
  * window.c: routines for the windows manipulations
  *
- * Copyright(c) 1997,1998,1999 - All Rights Reserved
+ * Copyright(c) 1997-2000 - All Rights Reserved
  *
  * See the COPYRIGHT file.
  */
 
 #ifndef lint
-static char rcsid[] = "@(#)$Id: window.c,v 1.64 1999/08/02 21:37:13 kalt Exp $";
+static char rcsid[] = "@(#)$Id: window.c,v 1.67 2000/07/31 23:28:28 kalt Exp $";
 #endif
 
 #include "os.h"
@@ -181,12 +181,14 @@ time_t		ts;
 	      int line_count = LNCNT(new);
 	      int i = 0;
 	      
-	      term_scroll(0, LI-3, line_count);
-	      while (i < line_count)
-		{
-		  term_move_cursor(0, LI-2-line_count+i);
-		  display_line(i++, new);
-		}
+	      if (term_scroll(0, LI-3, line_count) < 0)
+		sic_redowin(0);
+	      else
+		while (i < line_count)
+		  {
+		    term_move_cursor(0, LI-2-line_count+i);
+		    display_line(i++, new);
+		  }
 	    }
     }
   sic_logfile(new);
@@ -770,6 +772,27 @@ sic_wlist()
 	}
       if (buffer[0])
 	  vsic_slog(LOG_CLIENT, "---       %s", buffer);
+#if defined(DEBUG)
+      {
+	struct log_ *ltmp;
+	int total = 0, del = 0, delexp = 0;
+	time_t now = time(NULL);
+
+	for (ltmp = wtmp->flog; ltmp; ltmp = ltmp->nextl)
+	  {
+	    total += 1;
+	    if (ltmp->flags & LOG_DEL)
+	      {
+		del += 1;
+		if (now > ltmp->delts)
+		    delexp += 1;
+	      }
+	  }
+	vsic_slog(LOG_DEBUG,
+		  "--- Lines: %d total, %d to be deleted, %d expired",
+		  total, del, delexp);
+      }
+#endif
     }
 }
 
